@@ -1,12 +1,12 @@
-# $Id: TLPDB.pm 46421 2018-01-24 03:55:35Z preining $
-# TeXLive::TLPDB.pm - module for using tlpdb files
-# Copyright 2007-2017 Norbert Preining
+# $Id: TLPDB.pm 48144 2018-07-05 18:05:05Z karl $
+# TeXLive::TLPDB.pm - tlpdb plain text database files.
+# Copyright 2007-2018 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 
 package TeXLive::TLPDB;
 
-my $svnrev = '$Revision: 46421 $';
+my $svnrev = '$Revision: 48144 $';
 my $_modulerevision = ($svnrev =~ m/: ([0-9]+) /) ? $1 : "unknown";
 sub module_revision { return $_modulerevision; }
 
@@ -32,7 +32,6 @@ C<TeXLive::TLPDB> -- A database of TeX Live Packages
   $tlpdb->save;
   $tlpdb->media;
   $tlpdb->available_architectures();
-  $tlpdb->add_tlpcontainer($pkg, $ziploc [, $archrefs [, $dest ]] );
   $tlpdb->add_tlpobj($tlpobj);
   $tlpdb->needed_by($pkg);
   $tlpdb->remove_tlpobj($pkg);
@@ -92,12 +91,13 @@ C<TeXLive::TLPDB> -- A database of TeX Live Packages
 
 use TeXLive::TLConfig qw($CategoriesRegexp $DefaultCategory $InfraLocation
       $DatabaseName $MetaCategoriesRegexp $Archive
-      %TLPDBOptions %TLPDBSettings
+      $DefaultCompressorFormat %Compressors $CompressorExtRegexp
+      %TLPDBOptions %TLPDBSettings $ChecksumExtension
       $RelocPrefix $RelocTree);
 use TeXLive::TLCrypto;
 use TeXLive::TLPOBJ;
 use TeXLive::TLUtils qw(dirname mkdirhier member win32 info log debug ddebug
-                        tlwarn basename download_file merge_into tldie);
+                        tlwarn basename download_file merge_into tldie system_pipe);
 use TeXLive::TLWinGoo;
 
 use Cwd 'abs_path';
@@ -114,10 +114,10 @@ my $_listdir;
 
 C<< TeXLive::TLPDB->new >> creates a new C<TLPDB> object. If the
 argument C<root> is given it will be initialized from the respective
-location starting at $path. If C<$path> begins with C<http://> or
-C<ftp://>, the program C<wget> is used to download the file.  The
-C<$path> can also start with C<file:/> in which case it is treated as a
-file on the filesystem in the usual way.
+location starting at $path. If C<$path> begins with C<http://>, C<https://>,
+C<ftp://>, C<scp://>, C<ssh://> or C<I<user>@I<host>:>, the respective file
+is downloaded.  The C<$path> can also start with C<file:/> in which case it
+is treated as a file on the filesystem in the usual way.
 
 Returns an object of type C<TeXLive::TLPDB>, or undef if the root was
 given but no package could be read from that location.
@@ -281,6 +281,8 @@ sub from_file {
   my $rootpath = $self->root;
   if ($rootpath =~ m,https?://|ftp://,) {
     $media = 'NET';
+  } elsif ($rootpath =~ m,$TeXLive::TLUtils::SshURIRegex,) {
+    $media = 'NET';
   } else {
     if ($rootpath =~ m,file://*(.*)$,) {
       $rootpath = "/$1";
@@ -302,7 +304,7 @@ sub from_file {
   $self->{'media'} = $media;
   #
   # actually load the TLPDB
-  if ($path =~ m;^((https?|ftp)://|file:\/\/*);) {
+  if ($path =~ m;^((https?|ftp)://|file:\/\/*); || $path =~ m;$TeXLive::TLUtils::SshURIRegex;) {
     debug("TLPDB.pm: trying to initialize from $path\n");
     # now $xzfh filehandle is open, the file created
     # TLUtils::download_file will just overwrite what is there
@@ -312,54 +314,46 @@ sub from_file {
     ($tlpdbfh, $tlpdbfile) = TeXLive::TLUtils::tl_tmpfile();
     # same as above
     close($tlpdbfh);
-    my $tlpdbfile_quote = $tlpdbfile;
-    if (win32()) {
-      $tlpdbfile =~ s!/!\\!g;
-    }
-    $tlpdbfile_quote = "\"$tlpdbfile\"";
-    # if we have xzdec available we try the xz file
+    # if we have xz available we try the xz file
     my $xz_succeeded = 0 ;
-    if (defined($::progs{'xzdec'})) {
+    my $compressorextension = "<UNSET>";
+    if (defined($::progs{$DefaultCompressorFormat})) {
       # we first try the xz compressed file
       my ($xzfh, $xzfile) = TeXLive::TLUtils::tl_tmpfile();
       close($xzfh);
-      my $xzfile_quote = $xzfile;
-      if (win32()) {
-        $xzfile  =~ s!/!\\!g;
-      }
-      $xzfile_quote = "\"$xzfile\"";
-      my $xzdec = TeXLive::TLUtils::quotify_path_with_spaces($::progs{'xzdec'});
-      debug("trying to download $path.xz to $xzfile\n");
-      my $ret = TeXLive::TLUtils::download_file("$path.xz", "$xzfile");
+      my $decompressor = $::progs{$DefaultCompressorFormat};
+      $compressorextension = $Compressors{$DefaultCompressorFormat}{'extension'};
+      my @decompressorArgs = @{$Compressors{$DefaultCompressorFormat}{'decompress_args'}};
+      debug("trying to download $path.$compressorextension to $xzfile\n");
+      my $ret = TeXLive::TLUtils::download_file("$path.$compressorextension", "$xzfile");
       # better to check both, the return value AND the existence of the file
       if ($ret && (-r "$xzfile")) {
         # ok, let the fun begin
-        debug("xzdec-ing $xzfile to $tlpdbfile\n");
-        # xzdec *hopefully* returns 0 on success and anything else on failure
+        debug("decompressing $xzfile to $tlpdbfile\n");
+        # xz *hopefully* returns 0 on success and anything else on failure
         # we don't have to negate since not zero means error in the shell
         # and thus in perl true
-        if (system("$xzdec <$xzfile_quote >$tlpdbfile_quote")) {
-          debug("$xzdec $xzfile failed, trying plain file\n");
-          # to be sure we unlink the xz file and the tlpdbfile
-          unlink($xzfile);
+        if (!system_pipe($decompressor, $xzfile, $tlpdbfile, 1, @decompressorArgs)) {
+          debug("$decompressor $xzfile failed, trying plain file\n");
+          unlink($xzfile); # the above command only removes in case of success
         } else {
-          unlink($xzfile);
           $xz_succeeded = 1;
-          debug("found the uncompressed xz file\n");
+          debug("found the uncompressed $DefaultCompressorFormat file\n");
         }
       } 
     } else {
-      debug("no xzdec defined, not trying tlpdb.xz ...\n");
+      debug("no $DefaultCompressorFormat defined ...\n");
     }
     if (!$xz_succeeded) {
-      debug("TLPDB: downloading $path.xz didn't succeed, try $path\n");
+      debug("TLPDB: downloading $path.$compressorextension didn't succeed, try $path\n");
       my $ret = TeXLive::TLUtils::download_file($path, $tlpdbfile);
       # better to check both, the return value AND the existence of the file
-      if ($ret && (-r "$tlpdbfile")) {
+      if ($ret && (-r $tlpdbfile)) {
         # do nothing
       } else {
         unlink($tlpdbfile);
-        die "$0: open tlpdb($path) failed: $!";
+        tldie(  "$0: TLPDB::from_file could not download $path;\n"
+              . "$0: maybe the repository setting should be changed.\n");
       }
     }
     # if we are still here, then either the xz version was downloaded
@@ -368,7 +362,7 @@ sub from_file {
     #
     # before we open and proceed, verify the downloaded file
     if ($params{'verify'} && $media ne 'local_uncompressed') {
-      my ($r, $m) = TeXLive::TLCrypto::verify_checksum($tlpdbfile, "$path.$TeXLive::TLConfig::ChecksumExtension");
+      my ($r, $m) = TeXLive::TLCrypto::verify_checksum($tlpdbfile, "$path.$ChecksumExtension");
       if ($r == $VS_CHECKSUM_ERROR) {
         tldie("$0: checksum error when downloading $tlpdbfile from $path: $m\n");
       } elsif ($r == $VS_SIGNATURE_ERROR) {
@@ -376,7 +370,7 @@ sub from_file {
       } elsif ($r == $VS_CONNECTION_ERROR) {
         tldie("$0: cannot download: $m\n");
       } elsif ($r == $VS_UNSIGNED) {
-        debug("$0: remote database checksum is not signed, continuing anyway!\n");
+        debug("$0: remote database checksum is not signed, continuing anyway: $m\n");
         $self->verification_status($r);
       } elsif ($r == $VS_GPG_UNAVAILABLE) {
         debug("$0: TLPDB: no gpg available, continuing anyway!\n");
@@ -394,7 +388,7 @@ sub from_file {
     open($retfh, "<$tlpdbfile") || die "$0: open($tlpdbfile) failed: $!";
   } else {
     if ($params{'verify'} && $media ne 'local_uncompressed') {
-      my ($r, $m) = TeXLive::TLCrypto::verify_checksum($path, "$path.$TeXLive::TLConfig::ChecksumExtension");
+      my ($r, $m) = TeXLive::TLCrypto::verify_checksum($path, "$path.$ChecksumExtension");
       if ($r == $VS_CHECKSUM_ERROR) {
         tldie("$0: checksum error when downloading $path from $path: $m\n");
       } elsif ($r == $VS_SIGNATURE_ERROR) {
@@ -501,33 +495,33 @@ sub options_as_json {
   die("calling _as_json on virtual is not supported!") if ($self->is_virtual);
   my $opts = $self->options;
   my @opts;
-  for my $k (keys %TeXLive::TLConfig::TLPDBOptions) {
+  for my $k (keys %TLPDBOptions) {
     my %foo;
     $foo{'name'} = $k;
-    $foo{'tlmgrname'} = $TeXLive::TLConfig::TLPDBOptions{$k}[2];
-    $foo{'description'} = $TeXLive::TLConfig::TLPDBOptions{$k}[3];
-    $foo{'format'} = $TeXLive::TLConfig::TLPDBOptions{$k}[0];
-    $foo{'default'} = "$TeXLive::TLConfig::TLPDBOptions{$k}[1]";
-    # if ($TeXLive::TLConfig::TLPDBOptions{$k}[0] =~ m/^n/) {
+    $foo{'tlmgrname'} = $TLPDBOptions{$k}[2];
+    $foo{'description'} = $TLPDBOptions{$k}[3];
+    $foo{'format'} = $TLPDBOptions{$k}[0];
+    $foo{'default'} = "$TLPDBOptions{$k}[1]";
+    # if ($TLPDBOptions{$k}[0] =~ m/^n/) {
     #   if (exists($opts->{$k})) {
     #     $foo{'value'} = $opts->{$k};
     #     $foo{'value'} += 0;
     #   }
     #   $foo{'default'} += 0;
-    # } elsif ($TeXLive::TLConfig::TLPDBOptions{$k}[0] eq "b") {
+    # } elsif ($TLPDBOptions{$k}[0] eq "b") {
     #   if (exists($opts->{$k})) {
     #     $foo{'value'} = ($opts->{$k} ? TeXLive::TLUtils::True() : TeXLive::TLUtils::False());
     #   }
     #   $foo{'default'} = ($foo{'default'} ? TeXLive::TLUtils::True() : TeXLive::TLUtils::False());
     # } elsif ($k eq "location") {
     #   my %def;
-    #   $def{'main'} = $TeXLive::TLConfig::TLPDBOptions{$k}[1];
+    #   $def{'main'} = $TLPDBOptions{$k}[1];
     #   $foo{'default'} = \%def;
     #   if (exists($opts->{$k})) {
     #     my %repos = TeXLive::TLUtils::repository_to_array($opts->{$k});
     #     $foo{'value'} = \%repos;
     #   }
-    # } elsif ($TeXLive::TLConfig::TLPDBOptions{$k}[0] eq "p") {
+    # } elsif ($TLPDBOptions{$k}[0] eq "p") {
     #   # strings/path
     #   if (exists($opts->{$k})) {
     #     $foo{'value'} = $opts->{$k};
@@ -550,16 +544,16 @@ sub settings_as_json {
   die("calling _as_json on virtual is not supported!") if ($self->is_virtual);
   my $sets = $self->settings;
   my @json;
-  for my $k (keys %TeXLive::TLConfig::TLPDBSettings) {
+  for my $k (keys %TLPDBSettings) {
     my %foo;
     $foo{'name'} = $k;
-    $foo{'type'} = $TeXLive::TLConfig::TLPDBSettings{$k}[0];
-    $foo{'description'} = $TeXLive::TLConfig::TLPDBSettings{$k}[1];
-    # if ($TeXLive::TLConfig::TLPDBSettings{$k}[0] eq "b") {
+    $foo{'type'} = $TLPDBSettings{$k}[0];
+    $foo{'description'} = $TLPDBSettings{$k}[1];
+    # if ($TLPDBSettings{$k}[0] eq "b") {
     #   if (exists($sets->{$k})) {
     #     $foo{'value'} = ($sets->{$k} ? TeXLive::TLUtils::True() : TeXLive::TLUtils::False());
     #   }
-    # } elsif ($TeXLive::TLConfig::TLPDBSettings{$k} eq "available_architectures") {
+    # } elsif ($TLPDBSettings{$k} eq "available_architectures") {
     #   if (exists($sets->{$k})) {
     #     my @lof = $self->available_architectures;
     #     $foo{'value'} = \@lof;
@@ -628,9 +622,11 @@ sub save {
   open(FOO, ">$tmppath") || die "$0: open(>$tmppath) failed: $!";
   $self->writeout(\*FOO);
   close(FOO);
+  # on Windows the renaming sometimes fails, try to copy and unlink the
+  # .tmp file. This we do for all archs, cannot hurt.
   # if we managed that one, we move it over
-  die ("rename $tmppath to $path failed: $!")
-    unless rename($tmppath, $path);
+  TeXLive::TLUtils::copy ("-f", $tmppath, $path) or die ("copy $tmppath to $path failed: $!");
+  unlink ($tmppath) or tlwarn ("TLPDB: cannot unlink $tmppath: $!\n");
 }
 
 =pod
@@ -683,82 +679,6 @@ sub _available_architectures {
   return @archs;
 }
 
-=pod
-
-=item C<< $tlpdb->add_tlpcontainer($pkg, $ziploc [, $archrefs [, $dest ]] ) >>
-
-Installs the package C<$pkg> from the container files in C<$ziploc>. If
-C<$archrefs> is given then it must be a reference to a list of 
-architectures to be installed. If the normal (arch=all) package is
-architecture dependent then all arch packages in this list are installed.
-If C<$dest> is given then the files are
-installed into it, otherwise into the location of the TLPDB.
-
-Note that this procedure does NOT check for dependencies. So if your package
-adds new dependencies they are not necessarily fulfilled.
-
-=cut
-
-sub add_tlpcontainer {
-  my ($self, $package, $ziplocation, $archrefs, $dest) = @_;
-  if ($self->is_virtual) {
-    tlwarn("TLPDB: cannot add_tlpcontainer to a virtual tlpdb\n");
-    return 0;
-  }
-  my @archs;
-  if (defined($archrefs)) {
-    @archs = @$archrefs;
-  }
-  my $cwd = getcwd();
-  if ($ziplocation !~ m,^/,) {
-    $ziplocation = "$cwd/$ziplocation";
-  }
-  my $tlpobj = $self->_add_tlpcontainer($package, $ziplocation, "all", $dest);
-  if ($tlpobj->is_arch_dependent) {
-    foreach (@$archrefs) {
-      $self->_add_tlpcontainer($package, $ziplocation, $_, $dest);
-    }
-  }
-}
-
-sub _add_tlpcontainer {
-  my ($self, $package, $ziplocation, $arch, $dest) = @_;
-  my $unpackprog;
-  my $args;
-  # WARNING: If you change the location of the texlive.tlpdb this
-  # has to be changed, too!!
-  if (not(defined($dest))) { 
-    $dest = $self->{'root'};
-  }
-  my $container = "$ziplocation/$package";
-  if ($arch ne "all") {
-    $container .= ".$arch";
-  }
-  if (-r "$container.zip") {
-    $container .= ".zip";
-    $unpackprog="unzip";
-    $args="-o -qq $container -d $dest";
-  } elsif (-r "$container.xz") {
-    $container .= ".xz";
-    $unpackprog="NO_IDEA_HOW_TO_UNPACK_LZMA";
-    $args="NO IDEA WHAT ARGS IT NEEDS";
-    die "$0: xz checked for but not implemented, maybe update TLPDB.pm";
-  } else {
-    die "$0: No package $container (.zip or .xz) in $ziplocation";
-  }
-  tlwarn("TLPDB: Hmmm, this needs testing and error checking!\n");
-  tlwarn("Should we use -a -- adapt line endings etc?\n");
-  `$unpackprog $args`;
-  # we only create/add tlpobj for arch eq "all"
-  if ($arch eq "all") {
-    my $tlpobj = new TeXLive::TLPOBJ;
-    $tlpobj->from_file("$dest/$TeXLive::TLConfig::InfraLocation/tlpobj/$package.tlpobj");
-    $self->add_tlpobj($tlpobj);
-    return $tlpobj;
-  }
-}
-
-
 =pod
 
 =item C<< $tlpdb->get_package("pkgname") >> 
@@ -804,7 +724,7 @@ sub _get_package {
 
 =pod
 
-=item C<< $tlpdb->media_of_package($pkg [, $tag]);
+=item C<< $tlpdb->media_of_package($pkg [, $tag]) >>
 
 returns the media type of the package. In the virtual case a tag can
 be given and the media of that repository is used, otherwise the
@@ -874,7 +794,7 @@ sub list_packages {
     }
     # we have to be careful here: If a package
     # is only present in a subsidiary repository
-    # and the package is *not* explicitely
+    # and the package is *not* explicitly
     # pinned to it, it will not be installable.
     # This is what we want. But in this case
     # we don't want it to be listed by default.
@@ -939,6 +859,7 @@ sub expand_dependencies {
   my %install = ();
   my @archs = $totlpdb->available_architectures;
   for my $p (@_) {
+    next if ($p =~ m/^\s*$/);
     my ($pp, $aa) = split('@', $p);
     $install{$pp} = (defined($aa) ? $aa : 0);;
   }
@@ -951,7 +872,7 @@ sub expand_dependencies {
       next if ($p =~ m/^00texlive/);
       my $pkg = $self->get_package($p, ($install{$p}?$install{$p}:undef));
       if (!defined($pkg)) {
-        debug("W: $p is mentioned somewhere but not available, disabling\n");
+        ddebug("W: $p is mentioned somewhere but not available, disabling\n");
         $install{$p} = 0;
         next;
       }
@@ -1145,7 +1066,7 @@ sub _generate_listfile {
         push @lop, $d;
       }
     } else {
-      # speudo dependencies on $Package.ARCH can be ignored
+      # pseudo-dependencies on $Package.ARCH can be ignored
       if ($d !~ m/\.ARCH$/) {
         tlwarn("TLPDB: package $tlp->name depends on $d, but this does not exist\n");
       }
@@ -1179,7 +1100,7 @@ sub _generate_listfile {
     print TMP "*Title: ", $tlp->shortdesc, "\n";
     my $s = 0;
     # schemes size includes ONLY those packages which are directly
-    # included and direclty included files, not the size of the
+    # included and directly included files, not the size of the
     # included collections. But if a package is included in one of
     # the called for collections AND listed directly, we don't want
     # to count its size two times
@@ -1339,7 +1260,7 @@ sub verification_status {
 =item C<< $tlpdb->listdir >>
 
 The function C<listdir> allows to read and set the packages variable
-specifiying where generated list files are created.
+specifying where generated list files are created.
 
 =cut
 
@@ -1353,7 +1274,7 @@ sub listdir {
 
 =item C<< $tlpdb->config_src_container >>
 
-Returns 1 if the the texlive config option for src files splitting on 
+Returns 1 if the texlive config option for src files splitting on 
 container level is set. See Options below.
 
 =cut
@@ -1380,7 +1301,7 @@ sub config_src_container {
 
 =item C<< $tlpdb->config_doc_container >>
 
-Returns 1 if the the texlive config option for doc files splitting on 
+Returns 1 if the texlive config option for doc files splitting on 
 container level is set. See Options below.
 
 =cut
@@ -1755,7 +1676,7 @@ sub install_package_files {
     if ($opt_doc) { foreach ($tlpobj->docfiles) { push @installfiles, $_; } }
     # 
     # remove the RELOC prefix, but do NOT replace it with RelocTree
-    @installfiles = map { s!^$TeXLive::TLConfig::RelocPrefix/!!; $_; } @installfiles;
+    @installfiles = map { s!^$RelocPrefix/!!; $_; } @installfiles;
     # if the first argument of _install_data is scalar, it is the
     # place from where files should be installed
     if (!_install_data ($tmpdir, \@installfiles, $reloc, \@installfiles, $self)) {
@@ -1887,15 +1808,19 @@ sub not_virtual_install_package {
     if ($media eq 'local_uncompressed') {
       $container = \@installfiles;
     } elsif ($media eq 'local_compressed') {
-      if (-r "$root/$Archive/$pkg.tar.xz") {
-        $container = "$root/$Archive/$pkg.tar.xz";
-      } else {
-        tlwarn("TLPDB: cannot find package $pkg.tar.xz) in $root/$Archive\n");
-        next;
+      for my $ext (map { $Compressors{$_}{'extension'} } keys %Compressors) {
+        if (-r "$root/$Archive/$pkg.tar.$ext") {
+          $container = "$root/$Archive/$pkg.tar.$ext";
+        }
+      }
+      if (!$container) {
+        tlwarn("TLPDB: cannot find package $pkg.tar.$CompressorExtRegexp in $root/$Archive\n");
+        return(0);
       }
     } elsif (&media eq 'NET') {
-      $container = "$root/$Archive/$pkg.$TeXLive::TLConfig::DefaultContainerExtension";
+      $container = "$root/$Archive/$pkg.tar." . $Compressors{$DefaultCompressorFormat}{'extension'};
     }
+    debug("TLPDB::not_virtual_install_package: trying to install $container\n");
     $self->_install_data ($container, $reloc, \@installfiles, $totlpdb, $tlpobj->containersize, $tlpobj->containerchecksum)
       || return(0);
     # if we are installing from local_compressed or NET we have to fetch the respective
@@ -1914,13 +1839,13 @@ sub not_virtual_install_package {
       # - there are actually src/doc files present
       if ($container_src_split && $opt_src && $tlpobj->srcfiles) {
         my $srccontainer = $container;
-        $srccontainer =~ s/(\.tar\.xz|\.zip)$/.source$1/;
+        $srccontainer =~ s/\.tar\.$CompressorExtRegexp$/.source.tar.$1/;
         $self->_install_data ($srccontainer, $reloc, \@installfiles, $totlpdb, $tlpobj->srccontainersize, $tlpobj->srccontainerchecksum)
           || return(0);
       }
       if ($container_doc_split && $real_opt_doc && $tlpobj->docfiles) {
         my $doccontainer = $container;
-        $doccontainer =~ s/(\.tar\.xz|\.zip)$/.doc$1/;
+        $doccontainer =~ s/\.tar\.$CompressorExtRegexp$/.doc.tar.$1/;
         $self->_install_data ($doccontainer, $reloc, \@installfiles, $totlpdb, $tlpobj->doccontainersize, $tlpobj->doccontainerchecksum)
           || return(0);
       }
@@ -1930,8 +1855,8 @@ sub not_virtual_install_package {
       # in USER MODE that should NOT be done because we keep the information
       # there, but for now do it unconditionally
       if ($tlpobj->relocated) {
-        my $reloctree = $totlpdb->root . "/" . $TeXLive::TLConfig::RelocTree;
-        my $tlpkgdir = $reloctree . "/" . $TeXLive::TLConfig::InfraLocation;
+        my $reloctree = $totlpdb->root . "/" . $RelocTree;
+        my $tlpkgdir = $reloctree . "/" . $InfraLocation;
         my $tlpod = $tlpkgdir .  "/tlpobj";
         TeXLive::TLUtils::rmtree($tlpod) if (-d $tlpod);
         # we try to remove the tlpkg directory, that will succeed only
@@ -2013,14 +1938,6 @@ sub _install_data {
 
   my @filelist = @$filelistref;
 
-  # we assume that $::progs has been set up!
-  my $wget = $::progs{'wget'};
-  my $xzdec = TeXLive::TLUtils::quotify_path_with_spaces($::progs{'xzdec'});
-  if (!defined($wget) || !defined($xzdec)) {
-    tlwarn("TLPDB::_install_data: programs not set up properly; hmm.\n");
-    return(0);
-  }
-
   if (ref $what) {
     # determine the root from where we install
     # if the first argument $self is a string, then it should be the
@@ -2035,7 +1952,7 @@ sub _install_data {
     # if we are installing a reloc, add the RelocTree to the target
     if ($reloc) {
       if (!$totlpdb->setting("usertree")) {
-        $target .= "/$TeXLive::TLConfig::RelocTree";
+        $target .= "/$RelocTree";
       }
     }
 
@@ -2048,13 +1965,15 @@ sub _install_data {
     }
     # we always assume that copy will work
     return(1);
-  } elsif ($what =~ m,\.tar\.xz$,) {
+  } elsif ($what =~ m,\.tar\.$CompressorExtRegexp$,) {
     if ($reloc) {
       if (!$totlpdb->setting("usertree")) {
-        $target .= "/$TeXLive::TLConfig::RelocTree";
+        $target .= "/$RelocTree";
       }
     }
-    debug("tlpdb:_install_data: what=$what, target=$target, size=$whatsize, checksum=$whatcheck, tmpdir=$tempdir\n");
+    my $ww = ($whatsize || "<unset>");
+    my $ss = ($whatcheck || "<unset>");
+    debug("tlpdb:_install_data: what=$what, target=$target, size=$ww, checksum=$ss, tmpdir=$tempdir\n");
     my ($ret, $pkg) = TeXLive::TLUtils::unpack($what, $target, 'size' => $whatsize, 'checksum' => $whatcheck, 'tmpdir' => $tempdir);
     if (!$ret) {
       tlwarn("TLPDB::_install_package: $pkg\n");
@@ -2145,7 +2064,7 @@ sub remove_package {
           } else {
             # NO NOTHING HERE!!!
             # DON'T PUSH IT ON @goodfiles, it will be removed, which we do
-            # NOT want. We only want to supress the warning!
+            # NOT want. We only want to suppress the warning!
             push @debugfiles, $f;
           }
         } else {
@@ -2417,17 +2336,17 @@ sub setting {
 
 sub reset_options {
   my $self = shift;
-  for my $k (keys %TeXLive::TLConfig::TLPDBOptions) {
-    $self->option($k, $TeXLive::TLConfig::TLPDBOptions{$k}->[1]);
+  for my $k (keys %TLPDBOptions) {
+    $self->option($k, $TLPDBOptions{$k}->[1]);
   }
 }
 
 sub add_default_options {
   my $self = shift;
-  for my $k (sort keys %TeXLive::TLConfig::TLPDBOptions) {
+  for my $k (sort keys %TLPDBOptions) {
     # if the option is not set already, do set it to defaults
     if (! $self->option($k) ) {
-      $self->option($k, $TeXLive::TLConfig::TLPDBOptions{$k}->[1]);
+      $self->option($k, $TLPDBOptions{$k}->[1]);
     }
   }
 }
@@ -2604,7 +2523,9 @@ The purpose of virtual databases is to collect several data sources
 and present them in one way. The normal functions will always return
 the best candidate for the set of functions.
 
-More docs to be written
+More docs to be written someday, maybe.
+
+=over 4
 
 =cut
 
@@ -2839,7 +2760,7 @@ sub virtual_pinning {
 # means that:
 # for package "foo" the revision numbers of "foo" in the repos "repo1",
 # "repo2", and "main" are numerically compared and biggest number wins.
-# for all other packages of "repo1" and "repo2", other other repositories
+# for all other packages of "repo1" and "repo2", other repositories
 # are not considered.
 #
 # NOT IMPLEMENTED YET!!!
@@ -2983,6 +2904,8 @@ sub match_glob {
 
 =pod
 
+=back
+
 =head1 OPTIONS
 
 Options regarding the full TeX Live installation to be described are saved
