@@ -22,36 +22,37 @@ use open ':std', ':encoding(UTF-8)';
 
 # gain access to subroutines in the following modules
 use LatexIndent::Switches qw/storeSwitches %switches $is_m_switch_active $is_t_switch_active $is_tt_switch_active/;
-use LatexIndent::LogFile qw/logger output_logfile processSwitches/;
-use LatexIndent::GetYamlSettings qw/readSettings modify_line_breaks_settings get_indentation_settings_for_this_object get_every_or_custom_value get_indentation_information get_object_attribute_for_indentation_settings alignment_at_ampersand_settings %masterSettings/;
+use LatexIndent::LogFile qw/processSwitches $logger/;
+use LatexIndent::GetYamlSettings qw/yaml_read_settings yaml_modify_line_breaks_settings yaml_get_indentation_settings_for_this_object yaml_poly_switch_get_every_or_custom_value yaml_get_indentation_information yaml_get_object_attribute_for_indentation_settings yaml_alignment_at_ampersand_settings yaml_get_textwrap_removeparagraphline_breaks %masterSettings yaml_get_columns/;
 use LatexIndent::FileExtension qw/file_extension_check/;
 use LatexIndent::BackUpFileProcedure qw/create_back_up_file/;
 use LatexIndent::BlankLines qw/protect_blank_lines unprotect_blank_lines condense_blank_lines/;
-use LatexIndent::ModifyLineBreaks qw/modify_line_breaks_body modify_line_breaks_end remove_line_breaks_begin adjust_line_breaks_end_parent max_char_per_line paragraphs_on_one_line construct_paragraph_reg_exp/;
+use LatexIndent::ModifyLineBreaks qw/modify_line_breaks_body modify_line_breaks_end remove_line_breaks_begin adjust_line_breaks_end_parent text_wrap remove_paragraph_line_breaks construct_paragraph_reg_exp one_sentence_per_line text_wrap_remove_paragraph_line_breaks/;
 use LatexIndent::TrailingComments qw/remove_trailing_comments put_trailing_comments_back_in add_comment_symbol construct_trailing_comment_regexp/;
 use LatexIndent::HorizontalWhiteSpace qw/remove_trailing_whitespace remove_leading_space/;
-use LatexIndent::Indent qw/indent wrap_up_statement determine_total_indentation indent_begin indent_body indent_end_statement final_indentation_check  get_surrounding_indentation indent_children_recursively check_for_blank_lines_at_beginning put_blank_lines_back_in_at_beginning add_surrounding_indentation_to_begin_statement/;
+use LatexIndent::Indent qw/indent wrap_up_statement determine_total_indentation indent_begin indent_body indent_end_statement final_indentation_check  get_surrounding_indentation indent_children_recursively check_for_blank_lines_at_beginning put_blank_lines_back_in_at_beginning add_surrounding_indentation_to_begin_statement post_indentation_check/;
 use LatexIndent::Tokens qw/token_check %tokens/;
 use LatexIndent::HiddenChildren qw/find_surrounding_indentation_for_children update_family_tree get_family_tree check_for_hidden_children/;
 use LatexIndent::AlignmentAtAmpersand qw/align_at_ampersand find_aligned_block/;
 use LatexIndent::DoubleBackSlash qw/dodge_double_backslash un_dodge_double_backslash/;
 
 # code blocks
-use LatexIndent::Verbatim qw/put_verbatim_back_in find_verbatim_environments find_noindent_block find_verbatim_commands  put_verbatim_commands_back_in/;
-use LatexIndent::Environment qw/find_environments/;
-use LatexIndent::IfElseFi qw/find_ifelsefi/;
+use LatexIndent::Verbatim qw/put_verbatim_back_in find_verbatim_environments find_noindent_block find_verbatim_commands  put_verbatim_commands_back_in find_verbatim_special/;
+use LatexIndent::Environment qw/find_environments $environmentBasicRegExp/;
+use LatexIndent::IfElseFi qw/find_ifelsefi construct_ifelsefi_regexp $ifElseFiBasicRegExp/;
+use LatexIndent::Else qw/check_for_else_statement/;
 use LatexIndent::Arguments qw/get_arguments_regexp find_opt_mand_arguments get_numbered_arg_regexp construct_arguments_regexp/;
 use LatexIndent::OptionalArgument qw/find_optional_arguments/;
 use LatexIndent::MandatoryArgument qw/find_mandatory_arguments get_mand_arg_reg_exp/;
 use LatexIndent::RoundBrackets qw/find_round_brackets/;
 use LatexIndent::Item qw/find_items construct_list_of_items/;
-use LatexIndent::Braces qw/find_commands_or_key_equals_values_braces/;
+use LatexIndent::Braces qw/find_commands_or_key_equals_values_braces $braceBracketRegExpBasic/;
 use LatexIndent::Command qw/construct_command_regexp/;
 use LatexIndent::KeyEqualsValuesBraces qw/construct_key_equals_values_regexp/;
 use LatexIndent::NamedGroupingBracesBrackets qw/construct_grouping_braces_brackets_regexp/;
 use LatexIndent::UnNamedGroupingBracesBrackets qw/construct_unnamed_grouping_braces_brackets_regexp/;
-use LatexIndent::Special qw/find_special construct_special_begin/;
-use LatexIndent::Heading qw/find_heading construct_headings_levels/;
+use LatexIndent::Special qw/find_special construct_special_begin $specialBeginAndBracesBracketsBasicRegExp $specialBeginBasicRegExp/;
+use LatexIndent::Heading qw/find_heading construct_headings_levels $allHeadingsRegexp/;
 use LatexIndent::FileContents qw/find_file_contents_environments_and_preamble/;
 use LatexIndent::Preamble;
 
@@ -63,6 +64,7 @@ sub new{
     my $invocant = shift;
     my $class = ref($invocant) || $invocant;
     my $self = {@_};
+    $logger->trace(${$masterSettings{logFilePreferences}}{showDecorationStartCodeBlockTrace}) if ${$masterSettings{logFilePreferences}}{showDecorationStartCodeBlockTrace};
     bless ($self,$class);
     return $self;
 }
@@ -71,7 +73,6 @@ sub latexindent{
     my $self = shift;
     $self->storeSwitches;
     $self->processSwitches;
-    $self->readSettings;
     $self->file_extension_check;
     $self->operate_on_file;
 }
@@ -87,12 +88,14 @@ sub operate_on_file{
     $self->find_aligned_block;
     $self->remove_trailing_comments;
     $self->find_verbatim_environments;
-    $self->max_char_per_line;
+    $self->find_verbatim_special;
+    $self->text_wrap if ($is_m_switch_active and !${$masterSettings{modifyLineBreaks}{textWrapOptions}}{perCodeBlockBasis} and ${$masterSettings{modifyLineBreaks}{textWrapOptions}}{columns}>1);
     $self->protect_blank_lines;
     $self->remove_trailing_whitespace(when=>"before");
     $self->find_file_contents_environments_and_preamble;
     $self->dodge_double_backslash;
     $self->remove_leading_space;
+    $self->one_sentence_per_line if $is_m_switch_active;
     $self->process_body_of_text;
     $self->remove_trailing_whitespace(when=>"after");
     $self->condense_blank_lines;
@@ -102,13 +105,13 @@ sub operate_on_file{
     $self->put_trailing_comments_back_in;
     $self->put_verbatim_commands_back_in;
     $self->output_indented_text;
-    $self->output_logfile;
     return
 }
 
 sub construct_regular_expressions{
     my $self = shift;
     $self->construct_trailing_comment_regexp;
+    $self->construct_ifelsefi_regexp;
     $self->construct_list_of_items;
     $self->construct_special_begin;
     $self->construct_headings_levels;
@@ -123,25 +126,32 @@ sub construct_regular_expressions{
 sub output_indented_text{
     my $self = shift;
 
-    # output to screen, unless silent mode
-    print ${$self}{body} unless $switches{silentMode};
-
-    $self->logger("Output routine",'heading');
+    $logger->info("*Output routine:");
 
     # if -overwrite is active then output to original fileName
     if($switches{overwrite}) {
-        $self->logger("Overwriting file ${$self}{fileName}");
+        $logger->info("Overwriting file ${$self}{fileName}");
         open(OUTPUTFILE,">",${$self}{fileName});
         print OUTPUTFILE ${$self}{body};
         close(OUTPUTFILE);
     } elsif($switches{outputToFile}) {
-        $self->logger("Outputting to file $switches{outputToFile}");
+        $logger->info("Outputting to file $switches{outputToFile}");
         open(OUTPUTFILE,">",$switches{outputToFile});
         print OUTPUTFILE ${$self}{body};
         close(OUTPUTFILE);
     } else {
-        $self->logger("Not outputting to file; see -w and -o switches for more options.");
+        $logger->info("Not outputting to file; see -w and -o switches for more options.");
     }
+    
+    # put the final line in the logfile
+    $logger->info("${$masterSettings{logFilePreferences}}{endLogFileWith}") if ${$masterSettings{logFilePreferences}}{endLogFileWith};
+    
+    # github info line
+    $logger->info("*Please direct all communication/issues to:\nhttps://github.com/cmhughes/latexindent.pl") if ${$masterSettings{logFilePreferences}}{showGitHubInfoFooter};
+    
+    # output to screen, unless silent mode
+    print ${$self}{body} unless $switches{silentMode};
+
     return;
 }
 
@@ -149,19 +159,19 @@ sub process_body_of_text{
     my $self = shift;
 
     # find objects recursively
-    $self->logger('Phase 1: searching for objects','heading');
+    $logger->info('*Phase 1: searching for objects');
     $self->find_objects;
 
     # find all hidden child
-    $self->logger('Phase 2: finding surrounding indentation','heading');
+    $logger->info('*Phase 2: finding surrounding indentation');
     $self->find_surrounding_indentation_for_children;
 
     # indentation recursively
-    $self->logger('Phase 3: indenting objects','heading');
+    $logger->info('*Phase 3: indenting objects');
     $self->indent_children_recursively;
 
     # final indentation check
-    $self->logger('Phase 4: final indentation check','heading');
+    $logger->info('*Phase 4: final indentation check');
     $self->final_indentation_check;
 
     return;
@@ -171,51 +181,76 @@ sub find_objects{
     my $self = shift;
 
     # search for environments
-    $self->logger('looking for ENVIRONMENTS');
-    $self->find_environments;
+    $logger->trace('looking for ENVIRONMENTS') if $is_t_switch_active;
+    $self->find_environments if ${$self}{body} =~ m/$environmentBasicRegExp/s;
 
     # search for ifElseFi blocks
-    $self->logger('looking for IFELSEFI');
-    $self->find_ifelsefi;
+    $logger->trace('looking for IFELSEFI') if $is_t_switch_active;
+    $self->find_ifelsefi if ${$self}{body} =~ m/$ifElseFiBasicRegExp/s;
 
     # search for headings (part, chapter, section, setc)
-    $self->logger('looking for HEADINGS (chapter, section, part, etc)');
-    $self->find_heading;
+    $logger->trace('looking for HEADINGS (chapter, section, part, etc)') if $is_t_switch_active;
+    $self->find_heading if ${$self}{body} =~ m/$allHeadingsRegexp/s;
+
+    # the ordering of finding commands and special code blocks can change
+    $self->find_commands_or_key_equals_values_braces_and_special if ${$self}{body} =~ m/$specialBeginAndBracesBracketsBasicRegExp/s;
     
-    # search for commands with arguments
-    $self->logger('looking for COMMANDS and key = {value}');
-    $self->find_commands_or_key_equals_values_braces;
-
-    # search for special begin/end
-    $self->logger('looking for SPECIAL begin/end');
-    $self->find_special;
-
     # documents without preamble need a manual call to the paragraph_one_line routine
     if ($is_m_switch_active and !${$self}{preamblePresent}){
-        ${$self}{removeParagraphLineBreaks} = ${$masterSettings{modifyLineBreaks}{removeParagraphLineBreaks}}{all}||${$masterSettings{modifyLineBreaks}{removeParagraphLineBreaks}}{masterDocument}||0;
-        $self->paragraphs_on_one_line ; 
+        $self->yaml_get_textwrap_removeparagraphline_breaks;
+
+        # call the remove_paragraph_line_breaks and text_wrap routines
+        if(${$masterSettings{modifyLineBreaks}{removeParagraphLineBreaks}}{beforeTextWrap}){
+            $self->remove_paragraph_line_breaks if ${$self}{removeParagraphLineBreaks};
+            $self->text_wrap if (${$self}{textWrapOptions} and ${$masterSettings{modifyLineBreaks}{textWrapOptions}}{perCodeBlockBasis});
+        } else {
+            $self->text_wrap if (${$self}{textWrapOptions} and ${$masterSettings{modifyLineBreaks}{textWrapOptions}}{perCodeBlockBasis});
+            $self->remove_paragraph_line_breaks if ${$self}{removeParagraphLineBreaks};
+        }
     }
 
     # if there are no children, return
     if(${$self}{children}){
-        $self->logger("Objects have been found.",'heading');
+        $logger->trace("*Objects have been found.") if $is_t_switch_active;
     } else {
-        $self->logger("No objects found.");
+        $logger->trace("No objects found.");
         return;
     }
 
     # logfile information
-    $self->logger(Dumper(\%{$self}),'ttrace') if($is_tt_switch_active);
-    $self->logger("Operating on: ${$self}{name}",'heading');
-    $self->logger("Number of children:",'heading');
-    $self->logger(scalar (@{${$self}{children}}));
+    $logger->trace(Dumper(\%{$self})) if($is_tt_switch_active);
 
     return;
 }
 
+sub find_commands_or_key_equals_values_braces_and_special{
+    my $self = shift;
+
+    # the order in which we search for specialBeginEnd and commands/key/braces
+    # can change depending upon specialBeforeCommand
+    if(${$masterSettings{specialBeginEnd}}{specialBeforeCommand}){
+        # search for special begin/end
+        $logger->trace('looking for SPECIAL begin/end *before* looking for commands (see specialBeforeCommand)') if $is_t_switch_active;
+        $self->find_special if ${$self}{body} =~ m/$specialBeginBasicRegExp/s;
+
+        # search for commands with arguments
+        $logger->trace('looking for COMMANDS and key = {value}') if $is_t_switch_active;
+        $self->find_commands_or_key_equals_values_braces if ${$self}{body} =~ m/$braceBracketRegExpBasic/s;
+    } else {
+        # search for commands with arguments
+        $logger->trace('looking for COMMANDS and key = {value}') if $is_t_switch_active;
+        $self->find_commands_or_key_equals_values_braces if ${$self}{body} =~ m/$braceBracketRegExpBasic/s;
+
+        # search for special begin/end
+        $logger->trace('looking for SPECIAL begin/end') if $is_t_switch_active;
+        $self->find_special if ${$self}{body} =~ m/$specialBeginBasicRegExp/s;
+    }
+    return;
+}
+
 sub tasks_particular_to_each_object{
     my $self = shift;
-    $self->logger("There are no tasks particular to ${$self}{name}") if $is_t_switch_active;
+    $logger->trace("There are no tasks particular to ${$self}{name}") if $is_t_switch_active;
 }
 
 sub get_settings_and_store_new_object{
@@ -229,10 +264,15 @@ sub get_settings_and_store_new_object{
       
     # tasks particular to each object
     $latexIndentObject->tasks_particular_to_each_object;
+    
+    # removeParagraphLineBreaks and textWrapping fun!
+    $latexIndentObject->text_wrap_remove_paragraph_line_breaks if($is_m_switch_active);
 
     # store children in special hash
     push(@{${$self}{children}},$latexIndentObject);
 
+    # possible decoration in log file 
+    $logger->trace(${$masterSettings{logFilePreferences}}{showDecorationFinishCodeBlockTrace}) if ${$masterSettings{logFilePreferences}}{showDecorationFinishCodeBlockTrace};
 }
 
 sub tasks_common_to_each_object{
@@ -243,12 +283,12 @@ sub tasks_common_to_each_object{
 
     # update/create the ancestor information
     if($parent{ancestors}){
-      $self->logger("Ancestors *have* been found for ${$self}{name}") if($is_t_switch_active);
+      $logger->trace("Ancestors *have* been found for ${$self}{name}") if($is_t_switch_active);
       push(@{${$self}{ancestors}},@{$parent{ancestors}});
     } else {
-      $self->logger("No ancestors found for ${$self}{name}") if($is_t_switch_active);
+      $logger->trace("No ancestors found for ${$self}{name}") if($is_t_switch_active);
       if(defined $parent{id} and $parent{id} ne ''){
-        $self->logger("Creating ancestors with $parent{id} as the first one") if($is_t_switch_active);
+        $logger->trace("Creating ancestors with $parent{id} as the first one") if($is_t_switch_active);
         push(@{${$self}{ancestors}},{ancestorID=>$parent{id},ancestorIndentation=>\$parent{indentation},type=>"natural",name=>${$self}{name}});
       }
     }
@@ -267,7 +307,7 @@ sub tasks_common_to_each_object{
     ${$self}{bodyLineBreaks} = $bodyLineBreaks;
 
     # get settings for this object
-    $self->get_indentation_settings_for_this_object;
+    $self->yaml_get_indentation_settings_for_this_object;
 
     # give unique id
     $self->create_unique_id;
@@ -275,6 +315,14 @@ sub tasks_common_to_each_object{
     # add trailing text to the id to stop, e.g LATEX-INDENT-ENVIRONMENT1 matching LATEX-INDENT-ENVIRONMENT10
     ${$self}{id} .= $tokens{endOfToken};
 
+    # text wrapping can make the ID split across lines
+    ${$self}{idRegExp} = ${$self}{id};
+
+    if($is_m_switch_active){
+        my $IDwithLineBreaks = join("\\R?\\h*",split(//,${$self}{id}));
+        ${$self}{idRegExp} = qr/$IDwithLineBreaks/s;  
+    }
+
     # the replacement text can be just the ID, but the ID might have a line break at the end of it
     $self->get_replacement_text;
 
@@ -283,13 +331,13 @@ sub tasks_common_to_each_object{
     $self->adjust_replacement_text_line_breaks_at_end;
 
     # modify line breaks on body and end statements
-    $self->modify_line_breaks_body;
+    $self->modify_line_breaks_body if $is_m_switch_active;
 
     # modify line breaks end statements
-    $self->modify_line_breaks_end;
+    $self->modify_line_breaks_end if $is_m_switch_active;
 
     # check the body for current children
-    $self->check_for_hidden_children;
+    $self->check_for_hidden_children if ${$self}{body} =~ m/$tokens{beginOfToken}/;
 
     return;
 }
@@ -307,7 +355,7 @@ sub adjust_replacement_text_line_breaks_at_end{
 
     # the above regexp, when used below, will remove the trailing linebreak in ${$self}{linebreaksAtEnd}{end}
     # so we compensate for it here
-    $self->logger("Putting linebreak after replacementText for ${$self}{name}") if($is_t_switch_active);
+    $logger->trace("Putting linebreak after replacementText for ${$self}{name}") if($is_t_switch_active);
     if(defined ${$self}{horizontalTrailingSpace}){
         ${$self}{replacementText} .= ${$self}{horizontalTrailingSpace} unless(!${$self}{endImmediatelyFollowedByComment} and defined ${$self}{EndFinishesWithLineBreak} and ${$self}{EndFinishesWithLineBreak}==2);
     }
@@ -324,7 +372,7 @@ sub count_body_line_breaks{
     my $bodyLineBreaks = 0;
     $bodyLineBreaks++ while(${$self}{body} =~ m/\R/sxg);
     ${$self}{bodyLineBreaks} = $bodyLineBreaks;
-    $self->logger("bodyLineBreaks ${$self}{bodyLineBreaks}")  if((${$self}{bodyLineBreaks} != $oldBodyLineBreaks) and  $is_t_switch_active);
+    $logger->trace("bodyLineBreaks ${$self}{bodyLineBreaks}")  if((${$self}{bodyLineBreaks} != $oldBodyLineBreaks) and  $is_t_switch_active);
 }
 
 sub wrap_up_tasks{
@@ -334,10 +382,10 @@ sub wrap_up_tasks{
     my $child = @{${$self}{children}}[-1];
 
     # check if the last object was the last thing in the body, and if it has adjusted linebreaks
-    $self->adjust_line_breaks_end_parent;
+    $self->adjust_line_breaks_end_parent if $is_m_switch_active;
 
-    $self->logger(Dumper(\%{$child})) if($is_tt_switch_active);
-    $self->logger("replaced with ID: ${$child}{id}") if $is_t_switch_active;
+    $logger->trace(Dumper(\%{$child})) if($is_tt_switch_active);
+    $logger->trace("replaced with ID: ${$child}{id}") if $is_t_switch_active;
 
 }
 
